import { computed, onMounted, onUnmounted, onUpdated, reactive, ref, unref, watch, defineComponent } from 'vue'
import '/@/views/index.scss'
import logo from '/@/assets/logo.png'
import { useStore } from 'vuex'
import { Button, Input, Row, Col, Table, Pagination } from 'ant-design-vue'
import Child from './Child'
import VuexDom from './VuexDom'
import TableHelper from '/@/helper/tableHelper'
import { PAGE_SIZE_OPTIONS } from '/@/config'

const Home = defineComponent({
  setup() {
    // composition-api的基本用法
    const count = ref(0)
    const countR = reactive({val: 0})

    // computed/watch
    const computedSun = computed(() => unref(count) + countR.val)
    const watchProduct = ref(0)
    const stopWatch = watch([count, countR], (newValues, prevValues) => {
      watchProduct.value = newValues[0] * newValues[1].val
    })

    // 生命周期钩子
    console.log('setup')
    onMounted(() => {
      console.log('onMounted')
    })
    onUpdated(() => {
      console.log('onUpdated')
    })
    onUnmounted(() => {
      console.log('onUnmounted')
    })

    // 组件间数据传输props/emit/slot
    const childName = ref('子组件名称')

    // VUEX
    const VuexName = ref('Vuex名称')
    watch(VuexName, (newValues) => {
      store.dispatch('Demo/dispatchName', newValues)
    })
    const store = useStore()

    // helper
    const demoTableRequesFn = (obj) => {
      const { pageSize, pageNo } = obj
      let dataArr = [
        {
          tenantId: 1,
          companyName: '科大讯飞'
        },
        {
          tenantId: 2,
          companyName: '企业数字化'
        }
      ]
      return new Promise((resolve, reject) => {
        let arr = dataArr.slice(pageSize * (pageNo - 1), pageSize * pageNo)
        resolve({
          total: dataArr.length,
          records: arr
        })
      })
    }
    const TableHooks = new TableHelper(demoTableRequesFn)
    const tableColumns = [
      {
        title: '租户id',
        dataIndex: 'tenantId',
        key: 'tenantId',
        fixed: 'left'
      },
      {
        title: '租户名称',
        dataIndex: 'companyName',
        key: 'companyName',
      }
    ]

    const IntroFn = () => {
      return (
        <div class="demo-box">
          <h2>#此demo只介绍vue3，vuex4，vue-router4基本功能和用法</h2>
          <p>为了更好地理解，需要自行使用<a href="https://cn.vitejs.dev/" target="_blank">vite2</a>搭建环境，查询<a href="https://v3.cn.vuejs.org/" target="_blank">vue3官方文档</a>，与本教程配合使用</p>
        </div>
      )
    }
    const BasicApi = () => {
      return (
        <div class="demo-box">
          <h2>#composition-api的基本用法</h2>
          <div class="demo-content">
            <h3>#ref/reactive</h3>
            <div class="demo-content_intro">
              count(ref): {unref(count)}
              <Button style="margin-left: 50px;" onClick={() => count.value++}>点我count+1</Button>
            </div>
            <div class="demo-content_intro">
              countR.val(reactive): {countR.val}
              <Button style="margin-left: 50px;" onClick={() => countR.val++}>点我countR.val+1</Button>
            </div>
            <div class="demo-tips">关于ref和reactive的用法区别，以及各自适用情形，不做介绍，可自行查询文档</div>
            <div class="demo-tips">ref和reactive相关的api还有unref,toRef,isRef,isProxy,isReactive等（文档均有详细介绍）</div>
          </div>
          <div class="demo-content">
            <h3>#computed/watch</h3>
            <div class="demo-content_intro">
              count+countR.val(computed): {computedSun.value}
            </div>
            <div class="demo-content_intro">
              count*countR.val(watch): {unref(watchProduct)}
              <Button style="margin-left: 50px;" onClick={() => stopWatch()}>点击停止监听</Button>
            </div>
            <div class="demo-tips">
              computed和watch是常用的监听数据api，其中watch有停止侦听，
              清除副作用和选择副作用刷新时机的钩子。watchEffect和watch公用这些钩子，
              至于两者的区别在文档里有提及，这里不做过多赘述。
            </div>
          </div>
          <div class="demo-content">
            <h3>#生命周期钩子</h3>
            <div>
              你可以通过在生命周期钩子前面加上 “on” 来访问组件的生命周期钩子。
              本示例仅例举几个常用的钩子，可打开<strong>控制台</strong>查看执行顺序
            </div>
            <div class="demo-tips">
              注意：vue3中beforeCreate和created已标注为Not needed*
            </div>
          </div>
        </div>
      )
    }
    const DataFolw = () => {
      return (
        <div class="demo-box">
          <h2>#组件间数据传输props/emit/slot</h2>
          <div class="demo-content">
            <Row>
              <Col span="2">父组件输入框：</Col>
              <Col span="4">
                <Input v-model={[childName.value, "value"]}/>
              </Col>
            </Row>
            <Child name={unref(childName)} onHandleAdd={(res) => alert(res)} slotDom={() => <u style="margin-left: 20px;">来自父组件的u标签</u>}/>
            <div class="demo-tips">
              此处请留意antd-vue/Input在jsx中使用v-model的写法。在jsx/tsx中使用vue指令，详情请查看<a href="https://github.com/vitejs/vite/tree/main/packages/plugin-vue-jsx" target="_black">plugin-vue-jsx</a>
            </div>
          </div>
        </div>
      )
    }
    const Vuex = () => {
      return (
        <div class="demo-box">
          <h2>#VUEX</h2>
          <div class="demo-content">
            <Row>
              <Col span="2">父组件输入框：</Col>
              <Col span="4">
                <Input v-model={[VuexName.value, "value"]}/>
              </Col>
            </Row>
            <VuexDom/>
            <div class="demo-tips">
              vuex的使用网上有很多姿势，此事例采用的是useStore：<a href="https://next.vuex.vuejs.org/" target="_blank">官网</a>事例方式
            </div>
          </div>
        </div>
      )
    }
    const VueRouter = () => {
      return (
        <div class="demo-box">
          <h2>#Vuex-router</h2>
          <div class="demo-content">
            <div class="demo-tips">
              此demo采用<a href="https://next.router.vuejs.org/zh/introduction.html" target="_blank">vue-router4.x</a>。使用详情请阅读源码。
            </div>
          </div>
        </div>
      )
    }
    const Mixin = () => {
      return (
        <div class="demo-box">
          <h2>#Vue3中Mixin的姿势</h2>
          <div class="demo-content">
            <h3>#以常见的表格为例</h3>
            <div class="demo-content_intro">
              <Table
                columns={tableColumns}
                data-source={unref(TableHooks.tableData)}
                pagination={false}
                rowKey={(record) => record.tenantId}
              ></Table>
              <Pagination
                show-size-changer
                current={unref(TableHooks.currentPage)}
                pageSize={unref(TableHooks.pageSize)}
                total={unref(TableHooks.total)}
                pageSizeOptions={PAGE_SIZE_OPTIONS}
                onChange={(e) => TableHooks.onPageChange(e)}
                onShowSizeChange={(_current, size) => {
                  TableHooks.onPageSizeChange(size)
                }}
              />
            </div>
            <div class="demo-tips">
              这里的表格示例主要展示在vue3里如何进行代码逻辑的复用。
            </div>
          </div>
        </div>
      )
    }
    return () => {
      console.log('beforeRender')
      return (
        <div class="demo">
          {
            IntroFn()
          }
          {
            BasicApi()
          }
          {
            DataFolw()
          }
          {
            Vuex()
          }
          {
            VueRouter()
          }
          {
            Mixin()
          }
        </div>
      )
    }
  },
})

export default Home
