import { resolve } from 'path';
import { defineConfig } from 'vite'
import vue from '@vitejs/plugin-vue'
import vueJsx from '@vitejs/plugin-vue-jsx'
import styleImport from 'vite-plugin-style-import';
import { generateModifyVars } from './build/generateModifyVars';
import externalGlobals from "rollup-plugin-external-globals";
import commonjs from "rollup-plugin-commonjs";
// https://vitejs.dev/config/
export default defineConfig({
  css: {
    preprocessorOptions: {
      scss: {
        // 引入 var.scss 这样就可以在全局中使用 var.scss中预定义的变量了
        // 给导入的路径最后加上 ; 
        additionalData: '@import "./src/assets/scss/variables.scss";'
      },
      less: {
        javascriptEnabled: true,
        modifyVars: generateModifyVars(),
      },
    },
  },
  input: 'source/main.js',
  output: {
    file: 'build.js',
    format: 'esm'
  },
  plugins: [vue(), vueJsx(), styleImport({
    libs: [
      {
        libraryName: 'ant-design-vue',
        esModule: true,
        resolveStyle: (name) => {
          return `ant-design-vue/es/${name}/style/index`;
        },
      }
    ]
  })],
  base: './',
  resolve: {
    // 别名
    alias: {
      '/@': resolve(__dirname, './src')
    },
  },
  sourcemap: false,
  /**
   * 资源输出路径
   * @default 'assets'
   */
  assetsDir: 'assets',
  /**
   * esbuild转换目标。
   * @default 'es2019'
   */
  esbuildTarget: 'es2019',
  esbuild: {
    jsxFactory: 'h',
    jsxFragment: 'Fragment'
  },
  server: {
    port: 9999,
    proxy: {
      "/admin": "http://172.31.131.142:8100/"
    }
  },
  build: {
    outDir: 'dist',
    assetsDir: 'assets',
    sourcemap: false,
    rollupOptions: {
      external: ['vue', "ant-design-vue", "moment"],
      plugins: [
        commonjs(),
        externalGlobals({
          vue: "Vue",
          "ant-design-vue": "antd",
          moment: "moment",
        }),
      ],
    },
  },
})
